"""The intention of this file is to make the implementations of the core function fully customizable"""


####################################################################################################
###----------------------------------------Imports-----------------------------------------------###
####################################################################################################
import time
import cv2
import sys
import numpy
import os
import collections
import subprocess
import errno
import functools

from datetime import datetime
from PIL import ImageGrab
try:
    from PIL import Image
    from PIL import ImageOps
    from PIL import ImageDraw
    if sys.platform == 'win32': # TODO - Pillow now supports ImageGrab on macOS.
        from PIL import ImageGrab
    _PILLOW_UNAVAILABLE = False
except ImportError:
    # We ignore this because failures due to Pillow not being installed
    # should only happen when the functions that specifically depend on
    # Pillow are called. The main use case is when PyAutoGUI imports
    # PyScreeze, but Pillow isn't installed because the user is running
    # some platform/version of Python that Pillow doesn't support, then
    # importing PyAutoGUI should not automatically fail because it
    # imports PyScreeze.
    # So we have a `pass` statement here since a failure to import
    # Pillow shouldn't crash PyScreeze.
    _PILLOW_UNAVAILABLE = True

USE_IMAGE_NOT_FOUND_EXCEPTION = False

GRAYSCALE_DEFAULT = False

LARGE_FONT = ("Courier", 12)
NORM_FONT = ("Courier", 10)
SMALL_FONT = ("Courier", 8)

REGION_PICK_VIEW = 0
OFFSET_PICK_VIEW = 1
SIMILARITY_PICK_VIEW = 2
NAME_PICK_VIEW = 3
SW_MINIMIZE = 6


Box = collections.namedtuple('Box', 'left top width height')
Point = collections.namedtuple('Point', 'x y')
RGB = collections.namedtuple('RGB', 'red green blue')

GNOME_SCREENSHOT_EXISTS = False
try:
    if sys.platform not in ('java', 'darwin', 'win32'):
        whichProc = subprocess.Popen(
            ['which', 'gnome-screenshot'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        GNOME_SCREENSHOT_EXISTS = whichProc.wait() == 0
        print(f"{datetime.strftime(datetime.now(), '%H:%M:%S')}: gnome-screenshot installed: {GNOME_SCREENSHOT_EXISTS}")
except OSError as ex:
    if ex.errno == errno.ENOENT:
        # if there is no "which" program to find gnome-screenshot, then assume there
        # is no gnome-screenshot.
        pass
    else:
        raise

RUNNING_PYTHON_2 = sys.version_info[0] == 2

if not RUNNING_PYTHON_2:
    UniCode = str # On Python 3, all the isinstance(spam, (str, unicode)) calls will work the same as Python 2.
####################################################################################################
###----------------------------------------------------------------------------------------------###
####################################################################################################



####################################################################################################
###---------------------------------------Exceptions---------------------------------------------###
####################################################################################################
class PyScreezeException(Exception):
    """PyScreezeException is a generic exception class raised when a
    PyScreeze-related error happens. If a PyScreeze function raises an
    exception that isn't PyScreezeException or a subclass, assume it is
    a bug in PyScreeze."""
    pass

class ImageNotFoundException(PyScreezeException):
    """ImageNotFoundException is an exception class raised when the
    locate functions fail to locate an image. You must set
    pyscreeze.USE_IMAGE_NOT_FOUND_EXCEPTION to True to enable this feature.
    Otherwise, the locate functions will return None."""
    pass
####################################################################################################
###----------------------------------------------------------------------------------------------###
####################################################################################################





####################################################################################################
###----------------------------------------Functions---------------------------------------------###
####################################################################################################
def locate_on_screen(image, min_search_time=0, **kwargs):
    """TODO - rewrite this
    minSearchTime - amount of time in seconds to repeat taking
    screenshots and trying to locate a match.  The default of 0 performs
    a single search.
    """
    start = time.time()
    while True:
        try:
            screenshot_img = screenshot(region=None) # the locateAll() function must handle cropping to return accurate coordinates, so don't pass a region here.
            ret_val = locate(image, screenshot_img, **kwargs)
            try:
                screenshot_img.fp.close()
            except AttributeError:
                # Screenshots on Windows won't have an fp since they came from
                # ImageGrab, not a file. Screenshots on Linux will have fp set
                # to None since the file has been unlinked
                pass
            if ret_val or time.time() - start > min_search_time:
                return ret_val
        except ImageNotFoundException:
            if time.time() - start > min_search_time:
                if USE_IMAGE_NOT_FOUND_EXCEPTION:
                    raise
                return None

def locate(needle_image, haystack_image, **kwargs):
    """
    TODO
    """
    # Note: The gymnastics in this function is because we want to make sure to exhaust the iterator so that the needle and haystack files are closed in locateAll.
    kwargs['limit'] = 10
    points = list(locate_all(needle_image, haystack_image, **kwargs))
    if len(points) > 1:
        return points
    if len(points) == 1:
        return points[0]
    if USE_IMAGE_NOT_FOUND_EXCEPTION:
        raise ImageNotFoundException('Could not locate the image.')
    return None

def locate_all_opencv(needle_image, haystack_image, grayscale=None, limit=10000, region=None, step=1,
                      confidence=0.999):
    """
    TODO - rewrite this
        faster but more memory-intensive than pure python
        step 2 skips every other row and column = ~3x faster but prone to miss;
            to compensate, the algorithm automatically reduces the confidence
            threshold by 5% (which helps but will not avoid all misses).
        limitations:
          - OpenCV 3.x & python 3.x not tested
          - RGBA images are treated as RBG (ignores alpha channel)
    """
    if grayscale is None:
        grayscale = GRAYSCALE_DEFAULT

    confidence = float(confidence)

    needle_image = load_cv2(needle_image, grayscale)
    needle_height, needle_width = needle_image.shape[:2]
    haystack_image = load_cv2(haystack_image, grayscale)

    if region:
        haystack_image = haystack_image[region[1]:region[1]+region[3],
                                      region[0]:region[0]+region[2]]
    else:
        region = (0, 0)  # full image; these values used in the yield statement
    if (haystack_image.shape[0] < needle_image.shape[0] or
        haystack_image.shape[1] < needle_image.shape[1]):
        # avoid semi-cryptic OpenCV error below if bad size
        raise ValueError('needle dimension(s) exceed the haystack image or region dimensions')

    if step == 2:
        confidence *= 0.95
        needle_image = needle_image[::step, ::step]
        haystack_image = haystack_image[::step, ::step]
    else:
        step = 1

    # get all matches at once, credit: https://stackoverflow.com/questions/7670112/finding-a-subimage-inside-a-numpy-image/9253805#9253805
    result = cv2.matchTemplate(haystack_image, needle_image, cv2.TM_CCOEFF_NORMED)
    match_indices = numpy.arange(result.size)[(result > confidence).flatten()]
    matches = numpy.unravel_index(match_indices[:limit], result.shape)

    if len(matches[0]) == 0:
        if USE_IMAGE_NOT_FOUND_EXCEPTION:
            raise ImageNotFoundException('Could not locate the image (highest confidence = %.3f)' % result.max())
        return

    # use a generator for API consistency:
    matchx = matches[1] * step + region[0]  # vectorized
    matchy = matches[0] * step + region[1]
    for x, y in zip(matchx, matchy):
        yield Box(x, y, needle_width, needle_height)

def requires_pillow(wrapped_function):
    """
    A decorator that marks a function as requiring Pillow to be installed.
    This raises PyScreezeException if Pillow wasn't imported.
    """
    @functools.wraps(wrapped_function)
    def wrapper(*args, **kwargs):
        if _PILLOW_UNAVAILABLE:
            raise PyScreezeException('The Pillow package is required to use this function.')
        return wrapped_function(*args, **kwargs)
    return wrapper


@requires_pillow
def _screenshot_win32(image_filename=None, region=None):
    """
    TODO
    """
    # TODO - Use the winapi to get a screenshot, and compare performance with ImageGrab.grab()
    # https://stackoverflow.com/a/3586280/1893164
    img = ImageGrab.grab()
    if region is not None:
        assert len(region) == 4, 'region argument must be a tuple of four ints'
        region = [int(x) for x in region]
        img = img.crop((region[0], region[1], region[2] + region[0], region[3] + region[1]))
    if image_filename is not None:
        img.save(image_filename)
    return img


def _screenshot_osx(image_filename=None, region=None):
    """
    TODO
    """
    # TODO - use tmp name for this file.
    if image_filename is None:
        tmp_filename = 'screenshot%s.png' % (datetime.now().strftime('%Y-%m%d_%H-%M-%S-%f'))
    else:
        tmp_filename = image_filename

    tmp_filename_parts = tmp_filename.split("/")
    if tmp_filename_parts[-1].startswith("."):
        # the screencapture command does not work with hidden files, so
        # temporarily remove the dot from the filename, take screenshot and
        # rename file to hidden filename
        tmp_filename_parts[-1] = tmp_filename_parts[-1][1:]
        tmp_filename_unhidden = "/".join(tmp_filename_parts)
        subprocess.call(["screencapture", "-x", tmp_filename_unhidden])
        subprocess.call(["mv", tmp_filename_unhidden, tmp_filename])
    else:
        subprocess.call(["screencapture", "-x", tmp_filename])

    img = Image.open(tmp_filename)

    if region is not None:
        assert len(region) == 4, 'region argument must be a tuple of four ints'
        region = [int(x) for x in region]
        img = img.crop((region[0], region[1], region[2] + region[0], region[3] + region[1]))
        os.unlink(tmp_filename) # delete image of entire screen to save cropped version
        img.save(tmp_filename)
    else:
        # force loading before unlinking, Image.open() is lazy
        img.load()

    if image_filename is None:
        os.unlink(tmp_filename)
    return img


def _screenshot_linux(image_filename=None, region=None):
    """
    TODO
    """
    if not GNOME_SCREENSHOT_EXISTS:
        raise NotImplementedError('"gnome-screenshot" must be installed to use screenshot functions in Linux. Run: sudo apt-get install gnome-screenshot')
    if image_filename is None:
        tmp_filename = '.screenshot%s.png' % (datetime.now().strftime('%Y-%m%d_%H-%M-%S-%f'))
    else:
        tmp_filename = image_filename
        # print(tmp_filename)
    if GNOME_SCREENSHOT_EXISTS:
        subprocess.call(['gnome-screenshot', '-f', tmp_filename])
        img = Image.open(tmp_filename)

        if region is not None:
            assert len(region) == 4, 'region argument must be a tuple of four ints'
            region = [int(x) for x in region]
            img = img.crop((region[0], region[1], region[2] + region[0], region[3] + region[1]))
            os.unlink(tmp_filename) # delete image of entire screen to save cropped version
            img.save(tmp_filename)
        else:
            # force loading before unlinking, Image.open() is lazy
            img.load()

        if image_filename is None:
            os.unlink(tmp_filename)
        return img
    raise Exception('The gnome-screenshot program must be installed to take a screenshot with PyScreeze on Linux. Run: sudo apt-get install gnome-screenshot')




def load_cv2(img, grayscale=None):
    """
    TODO
    """
    # load images if given filename, or convert as needed to opencv
    # Alpha layer just causes failures at this point, so flatten to RGB.
    # RGBA: load with -1 * cv2.CV_LOAD_IMAGE_COLOR to preserve alpha
    # to matchTemplate, need template and image to be the same wrt having alpha

    if grayscale is None:
        grayscale = GRAYSCALE_DEFAULT
    if isinstance(img, (str, UniCode)):
        # The function imread loads an image from the specified file and
        # returns it. If the image cannot be read (because of missing
        # file, improper permissions, unsupported or invalid format),
        # the function returns an empty matrix
        # http://docs.opencv.org/3.0-beta/modules/imgcodecs/doc/reading_and_writing_images.html
        if grayscale:
            img_cv = cv2.imread(img, cv2.IMREAD_GRAYSCALE)
        else:
            img_cv = cv2.imread(img, cv2.IMREAD_COLOR)
        if img_cv is None:
            raise IOError("Failed to read %s because file is missing, "
                          "has improper permissions, or is an "
                          "unsupported or invalid format" % img)
    elif isinstance(img, numpy.ndarray):
        # don't try to convert an already-gray image to gray
        if grayscale and len(img.shape) == 3:  # and img.shape[2] == 3:
            img_cv = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        else:
            img_cv = img
    elif hasattr(img, 'convert'):
        # assume its a PIL.Image, convert to cv format
        img_array = numpy.array(img.convert('RGB'))
        img_cv = img_array[:, :, ::-1].copy()  # -1 does RGB -> BGR
        if grayscale:
            img_cv = cv2.cvtColor(img_cv, cv2.COLOR_BGR2GRAY)
    else:
        raise TypeError('expected an image filename, OpenCV numpy array, or PIL image')
    return img_cv

def _kmp(needle, haystack, _dummy): # Knuth-Morris-Pratt search algorithm implementation (to be used by screen capture)
    """
    TODO
    """
    # build table of shift amounts
    shifts = [1] * (len(needle) + 1)
    shift = 1
    for pos in range(len(needle)):
        while shift <= pos and needle[pos] != needle[pos-shift]:
            shift += shifts[pos-shift]
        shifts[pos+1] = shift

    # do the actual search
    start_pos = 0
    match_len = 0
    for elem in haystack:
        while match_len == len(needle) or \
              match_len >= 0 and needle[match_len] != elem:
            start_pos += shifts[match_len]
            match_len -= shifts[match_len]
        match_len += 1
        if match_len == len(needle):
            yield start_pos


def _stepping_find(needle, haystack, step):
    """
    TODO
    """
    for start_pos in range(0, len(haystack) - len(needle) + 1):
        found_match = True
        for pos in range(0, len(needle), step):
            if haystack[start_pos + pos] != needle[pos]:
                found_match = False
                break
        if found_match:
            yield start_pos


def center(coords):
    """
    Returns a `Point` object with the x and y set to an integer determined by the format of `coords`.
    The `coords` argument is a 4-integer tuple of (left, top, width, height).
    For example:
    >>> center((10, 10, 6, 8))
    Point(x=13, y=14)
    >>> center((10, 10, 7, 9))
    Point(x=13, y=14)
    >>> center((10, 10, 8, 10))
    Point(x=14, y=15)
    """

    # TODO - one day, add code to handle a Box namedtuple.
    return Point(coords[0] + int(coords[2] / 2), coords[1] + int(coords[3] / 2))


def pixel_matches_color(x, y, expected_rgb_color, tolerance=0):
    """
    TODO
    """
    pix = pixel(x, y)
    if len(pix) == 3 or len(expected_rgb_color) == 3: #RGB mode
        r, g, b = pix[:3]
        ex_r, ex_g, ex_b = expected_rgb_color[:3]
        return (abs(r - ex_r) <= tolerance) and (abs(g - ex_g) <= tolerance) and (abs(b - ex_b) <= tolerance)
    if (len(pix) == 4 and len(expected_rgb_color) == 4): #RGBA mode
        r, g, b, a = pix
        ex_r, ex_g, ex_b, ex_a = expected_rgb_color
        return (abs(r - ex_r) <= tolerance) and (abs(g - ex_g) <= tolerance) and (abs(b - ex_b) <= tolerance) and (abs(a - ex_a) <= tolerance)
    assert False, 'Color mode was expected to be length 3 (RGB) or 4 (RGBA), but pixel is length %s and expectedRGBColor is length %s' % (len(pix), len(expected_rgb_color))

def pixel(x, y):
    """
    TODO
    """
    # Need to select only the first three values of the color in
    # case the returned pixel has an alpha channel
    return RGB(*(screenshot().getpixel((x, y))[:3]))


# This makes sure that the openCV implementation of that function is used
locate_all = locate_all_opencv

if sys.platform.startswith('java'):
    raise NotImplementedError('Jython is not yet supported by PyScreeze.')
if sys.platform == 'darwin':
    screenshot = _screenshot_osx
elif sys.platform == 'win32':
    screenshot = _screenshot_win32
else: # TODO - Make this more specific. "Anything else" does not necessarily mean "Linux".
    screenshot = _screenshot_linux
####################################################################################################
###----------------------------------------------------------------------------------------------###
####################################################################################################



"""GUI for editing the screenshots"""
import argparse
from genericpath import isdir
import tkinter
import tkinter as tk
from tkinter import ttk
from tkinter.messagebox import askyesnocancel
from tkinter import filedialog as fd
import os
import glob
from PIL.PngImagePlugin import PngImageFile
from matplotlib.pyplot import show, text
from pyparsing import col
from aai_engine_package.screenshot_taker import MessageBox
from aai_engine_package import engine

DEV = False

MIN_WIDTH = 900
MIN_HEIGHT = 550
SHOW_IMAGE_PADDING = 300

NORMAL_SCREENSHOT = 0
OCR_SCREENSHOT = 1

class ApplicationData():
    """
    Class to keep track of application data and notify the listeners.
    """
    def __init__(self):
        self._directory = ""
        self._observers = []

    @property
    def directory(self):
        """return directory
        """
        return self._directory

    def _observed(func):
        def wrapper(self,value):
            func(self, value) # call setter function

            # notify all observers (initiate their callbacks)
            for callback in self._observers:
                callback(self._directory)

        return wrapper

    @directory.setter
    @_observed
    def directory(self, value):
        print(f"Setting new directory: {value}")
        self._directory = value

    def bind_to(self, callback):
        """bind to
        """
        self._observers.append(callback)


# END ApplicationData
# --- --- --- --- --- --- --- --- ---
# START GUI

class Toolbar(tk.Frame):
    """
    Class to manage the toolbar for application level funcitons.
    """
    def __init__(self, parent, *args, **kwargs):
        tk.Frame.__init__(self, parent, *args, **kwargs)
        self.parent = parent
        menubar = tk.Menu(self.parent.master)
        self.parent.master.config(menu=menubar)

        file_menu = tk.Menu(menubar, tearoff=0)
        file_menu.add_command(label="Open task folder", command=self.on_open)
        file_menu.add_command(label="New", command=self.on_new)
        file_menu.add_command(label="New OCR", command=self.on_new_ocr)
        file_menu.add_command(label="refresh", command=self.on_refresh)
        menubar.add_cascade(label="File", menu=file_menu)


    def on_open(self):
        """_summary_
        """
        directory = fd.askdirectory(title="Select a directory", master=self.parent)
        if isinstance(directory,str):
            self.parent.data.directory = directory

    def on_new(self):
        """_summary_
        """
        engine.main(self.parent.data.directory, update_function_after_edit=self.parent.load_files,)

    def on_new_ocr(self):
        """_summary_
        """
        engine.main(self.parent.data.directory, update_function_after_edit=self.parent.load_files, screenshot_type=OCR_SCREENSHOT)

    def on_refresh(self):
        """_summary_
        """
        self.parent.data.directory = self.parent.data.directory


class Main(tk.Frame):
    """
    Class to manage the main view of the application.
    """
    def __init__(self, parent, data, *args, **kwargs):
        tk.Frame.__init__(self, parent, *args, **kwargs)
        self.data = data
        self.button_width = 10
        self.lst = tk.Listbox(self, width=20)
        self.lst.pack(sid="left", fill=tk.BOTH, expand=0)
        self.lst.bind("<<ListboxSelect>>", self._show_img)
        self.parent = parent

        # edit_button = tk.Button(self, text="Edit", command = self._edit_img)
        edit_button = ttk.Button(self, text="Edit", command = self._edit_img, style='Accent.TButton', width = self.button_width)
        # edit_button.configure(width=10, background="#ffffff" ,activebackground="#33B5E5", relief = tk.FLAT)
        edit_button.pack(padx=(0, 25), pady=(25,0), side=tkinter.BOTTOM)

        delete_button = ttk.Button(self, text="Delete", command = self._delete_img, style= 'Accent.TButton', width = self.button_width)
        delete_button.pack(padx=(0, 25), pady=(25,0), side=tkinter.BOTTOM)

        self.canvas = tk.Canvas(self)
        self.canvas.pack()

        self.show_img_label = tk.Label(self, text="", )
        self.show_img_label.pack(side=tkinter.BOTTOM)


    # Observes application data (mainly current directory)
    def on_data_update(self, directory):
        """_summary_

        Args:
            directory (_type_): _description_
        """
        # Check if directory is valid
        if os.path.isdir(directory):
            self._load_files(directory)
        else:
            print(f"{directory} is not a valid directory!")

    def _load_files(self, directory):
        """_summary_

        Args:
            directory (_type_): _description_
        """
        self.lst.delete(0, tk.END) # clear previously opened folder list
        imgs = glob.glob(os.path.join(directory,'*.png'))
        for img in imgs:
            img = img.replace(directory + '/','')
            self.lst.insert(tk.END, img)

        if self.lst.size() > 0:
            self.lst.selection_set(0) # select first item of the list to show
            self._show_img(None) # show to selected item

    def _delete_img(self):
        """on click function for the deletion of an image
        """
        selec = self.lst.curselection()
        filename = os.path.join(self.data.directory, self.lst.get(selec))
        answer = askyesnocancel("Deleting image", f"Are you sure you want to delete {filename}?")
        if answer:
            os.remove(filename)
            print(f"Deleted image: {filename}")
            # Refresh the current directory
            self.on_data_update(self.data._directory)


    def _edit_img(self):
        """_summary_
        """
        selec = self.lst.curselection()
        try:
            filename = os.path.join(self.data.directory, self.lst.get(selec))
            print(f"Edit image: {filename}")
            img = PngImageFile(filename)
            img_keys = list(img.text.keys())
            if ('offset_x' not in img_keys) or ('offset_y' not in img_keys) or ('upper_confidence' not in img_keys) or ('lower_confidence' not in img_keys):
                print("  -The image does not contain the right metadata!")
                return

            if 'ocr_box_relative' in img.text:
                MessageBox(
                    self, self.parent,
                    "GUI - Message",
                    "You cannot edit OCR images because the OCR box could be placed outside the needle image!\nPlease create a new OCR image instead. (File -> new OCR)",
                    (650, 170)
                )
            else:
                engine.main(self.data.directory, update_function_after_edit=self._load_files, needle=filename, editing=True)
        except Exception as ex:
            print("Image could not be edited!")

    def process_ocr_box(self, ocr_box):
        """Processes the ocr_box metadata into readable information

        Args:
            ocr_box (str): ocr_box metadata

        Returns:
            str: The processed metadata
        """
        info = ocr_box[1:-1].split(',')
        result = "This is an OCR image. It has an OCR box attached to it, where you can perform OCR operations.\n"
        result += "The OCR box is drawn at the given offset, seen from the top left of the image \n"
        result += "The following metadata of the OCR box is available: \n"
        result += f"Offset=({info[0]},{info[1]}) - width: {info[2]}, height: {info[3]})"
        return result

    def _show_img(self, event):
        """_summary_

        Args:
            event (_type_): _description_
        """
        try:
            selec = self.lst.curselection()
            filename = os.path.join(self.data.directory, self.lst.get(selec))

            img = tk.PhotoImage(file=filename)
            metadata = PngImageFile(filename).text
            if 'ocr_box_relative' in metadata.keys():
                self.show_img_label.configure(text=self.process_ocr_box(metadata['ocr_box_relative']))
            else:
                self.show_img_label.configure(text="")
            width, height = img.width(), img.height()
            print(f"Currently showing: {filename}")
            self.canvas.image = img
            self.canvas.config(width=width, height=height)
            self.canvas.create_image(0, 0, image=img, anchor=tk.NW)
            self.canvas.pack()

            # Adjust screensize to image dimensions
            height = img.height()
            width = img.width()
            # Height setting:
            if (height + SHOW_IMAGE_PADDING) > MIN_HEIGHT:
                height += SHOW_IMAGE_PADDING
                if height > self.parent.parent.winfo_screenheight():
                    height = self.parent.parent.winfo_screenheight()
            else:
                height = MIN_HEIGHT
            # Width setting:
            if (width + SHOW_IMAGE_PADDING) > MIN_WIDTH:
                width += SHOW_IMAGE_PADDING
                if width > self.parent.parent.winfo_screenwidth():
                    width = self.parent.parent.winfo_screenwidth()
            else:
                width = MIN_WIDTH
            self.parent.parent.geometry(f"{str(width)}x{str(height)}")
            print(f"New window dimensions: ({str(width)},{str(height)})")

        except Exception as ex:
            print(ex)
            self.canvas.delete("all")
            print("Image could not be shown!")


class MainApplication(tk.Frame):
    """
    Class to group all UI elements.
    """
    def __init__(self, parent, command_line_options, *args, **kwargs):
        tk.Frame.__init__(self, parent, *args, **kwargs)
        self.master.title("AAI Image manager")

        self.parent = parent

        self.data = ApplicationData()

        self.toolbar = Toolbar(self)
        self.main = Main(self, self.data)

        self.data.bind_to(self.main.on_data_update) # main view observes the data

        self.toolbar.pack(side="top", fill="x")
        self.main.pack(side="right", fill="both", expand=True)

        parent.tk.call('source', engine._get_theme_path())
        parent.tk.call("set_theme", "dark")

        print("The command line arguments are:")
        for key,value in command_line_options.items():
            print(f"  -Name= {key}, Value= {value}")

        self.data.directory = command_line_options['d']

        # DEV ONLY
        if DEV:
            self.data.directory = r'/home/toto/aai/aai_engine_example_app/rpa_challenge_1_linux/img'

    def load_files(self, directory):
        """_summary_
        """
        self.main._load_files(directory)

def get_args():
    """Parser for the command line options of the GUI

    Returns:
        dict: Dictionary of the command line options
    """
    parser = argparse.ArgumentParser("Arguments for the GUI")

    # Add arguments here:
    parser.add_argument("-d",metavar="DIRECTORY", type=str, help= "Enter a directory where the application will look for png images",default=os.path.join(os.path.expanduser('~'),"Pictures"))

    return vars(parser.parse_args())

def main():
    """The main function for the GUI
    """
    args = get_args()
    root = tk.Tk()
    root.geometry("800x495+50+50")
    MainApplication(root,args).pack(side="top", fill="both", expand=True)
    root.mainloop()

if __name__ == "__main__":
    main()
